@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,15 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Callable, Dict, Optional, Sequence, Tuple
+from typing import Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import operations_v1  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+import google.auth  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 
 import grpc  # type: ignore
@@ -30,9 +28,8 @@ import grpc  # type: ignore
 from google.cloud.firestore_admin_v1.types import field
 from google.cloud.firestore_admin_v1.types import firestore_admin
 from google.cloud.firestore_admin_v1.types import index
-from google.longrunning import operations_pb2 as operations  # type: ignore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.longrunning import operations_pb2  # type: ignore
+from google.protobuf import empty_pb2  # type: ignore
 from .base import FirestoreAdminTransport, DEFAULT_CLIENT_INFO
 
 
@@ -56,7 +53,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
         self,
         *,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Sequence[str] = None,
         channel: grpc.Channel = None,
@@ -70,7 +67,8 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -181,7 +179,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
     def create_channel(
         cls,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -212,13 +210,15 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
             google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -245,7 +245,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
     @property
     def create_index(
         self,
-    ) -> Callable[[firestore_admin.CreateIndexRequest], operations.Operation]:
+    ) -> Callable[[firestore_admin.CreateIndexRequest], operations_pb2.Operation]:
         r"""Return a callable for the create index method over gRPC.
 
         Creates a composite index. This returns a
@@ -268,7 +268,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
             self._stubs["create_index"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/CreateIndex",
                 request_serializer=firestore_admin.CreateIndexRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["create_index"]
 
@@ -327,7 +327,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
     @property
     def delete_index(
         self,
-    ) -> Callable[[firestore_admin.DeleteIndexRequest], empty.Empty]:
+    ) -> Callable[[firestore_admin.DeleteIndexRequest], empty_pb2.Empty]:
         r"""Return a callable for the delete index method over gRPC.
 
         Deletes a composite index.
@@ -346,7 +346,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
             self._stubs["delete_index"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/DeleteIndex",
                 request_serializer=firestore_admin.DeleteIndexRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["delete_index"]
 
@@ -377,7 +377,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
     @property
     def update_field(
         self,
-    ) -> Callable[[firestore_admin.UpdateFieldRequest], operations.Operation]:
+    ) -> Callable[[firestore_admin.UpdateFieldRequest], operations_pb2.Operation]:
         r"""Return a callable for the update field method over gRPC.
 
         Updates a field configuration. Currently, field updates apply
@@ -411,7 +411,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
             self._stubs["update_field"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/UpdateField",
                 request_serializer=firestore_admin.UpdateFieldRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["update_field"]
 
@@ -453,7 +453,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
     @property
     def export_documents(
         self,
-    ) -> Callable[[firestore_admin.ExportDocumentsRequest], operations.Operation]:
+    ) -> Callable[[firestore_admin.ExportDocumentsRequest], operations_pb2.Operation]:
         r"""Return a callable for the export documents method over gRPC.
 
         Exports a copy of all or a subset of documents from
@@ -481,14 +481,14 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
             self._stubs["export_documents"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/ExportDocuments",
                 request_serializer=firestore_admin.ExportDocumentsRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["export_documents"]
 
     @property
     def import_documents(
         self,
-    ) -> Callable[[firestore_admin.ImportDocumentsRequest], operations.Operation]:
+    ) -> Callable[[firestore_admin.ImportDocumentsRequest], operations_pb2.Operation]:
         r"""Return a callable for the import documents method over gRPC.
 
         Imports documents into Google Cloud Firestore.
@@ -513,7 +513,7 @@ class FirestoreAdminGrpcTransport(FirestoreAdminTransport):
             self._stubs["import_documents"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/ImportDocuments",
                 request_serializer=firestore_admin.ImportDocumentsRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["import_documents"]
 
