@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,16 +13,15 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import grpc_helpers_async  # type: ignore
 from google.api_core import operations_v1  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
+import packaging.version
 
 import grpc  # type: ignore
 from grpc.experimental import aio  # type: ignore
@@ -31,9 +29,8 @@ from grpc.experimental import aio  # type: ignore
 from google.cloud.firestore_admin_v1.types import field
 from google.cloud.firestore_admin_v1.types import firestore_admin
 from google.cloud.firestore_admin_v1.types import index
-from google.longrunning import operations_pb2 as operations  # type: ignore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.longrunning import operations_pb2  # type: ignore
+from google.protobuf import empty_pb2  # type: ignore
 from .base import FirestoreAdminTransport, DEFAULT_CLIENT_INFO
 from .grpc import FirestoreAdminGrpcTransport
 
@@ -59,7 +56,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     def create_channel(
         cls,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -86,13 +83,15 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         Returns:
             aio.Channel: A gRPC AsyncIO channel object.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers_async.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -100,7 +99,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         self,
         *,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         channel: aio.Channel = None,
@@ -114,7 +113,8 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -173,7 +173,6 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
             self._ssl_channel_credentials = None
-
         else:
             if api_mtls_endpoint:
                 host = api_mtls_endpoint
@@ -252,7 +251,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     def create_index(
         self,
     ) -> Callable[
-        [firestore_admin.CreateIndexRequest], Awaitable[operations.Operation]
+        [firestore_admin.CreateIndexRequest], Awaitable[operations_pb2.Operation]
     ]:
         r"""Return a callable for the create index method over gRPC.
 
@@ -276,7 +275,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             self._stubs["create_index"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/CreateIndex",
                 request_serializer=firestore_admin.CreateIndexRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["create_index"]
 
@@ -338,7 +337,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     @property
     def delete_index(
         self,
-    ) -> Callable[[firestore_admin.DeleteIndexRequest], Awaitable[empty.Empty]]:
+    ) -> Callable[[firestore_admin.DeleteIndexRequest], Awaitable[empty_pb2.Empty]]:
         r"""Return a callable for the delete index method over gRPC.
 
         Deletes a composite index.
@@ -357,7 +356,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             self._stubs["delete_index"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/DeleteIndex",
                 request_serializer=firestore_admin.DeleteIndexRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["delete_index"]
 
@@ -391,7 +390,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     def update_field(
         self,
     ) -> Callable[
-        [firestore_admin.UpdateFieldRequest], Awaitable[operations.Operation]
+        [firestore_admin.UpdateFieldRequest], Awaitable[operations_pb2.Operation]
     ]:
         r"""Return a callable for the update field method over gRPC.
 
@@ -426,7 +425,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             self._stubs["update_field"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/UpdateField",
                 request_serializer=firestore_admin.UpdateFieldRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["update_field"]
 
@@ -470,7 +469,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     def export_documents(
         self,
     ) -> Callable[
-        [firestore_admin.ExportDocumentsRequest], Awaitable[operations.Operation]
+        [firestore_admin.ExportDocumentsRequest], Awaitable[operations_pb2.Operation]
     ]:
         r"""Return a callable for the export documents method over gRPC.
 
@@ -499,7 +498,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             self._stubs["export_documents"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/ExportDocuments",
                 request_serializer=firestore_admin.ExportDocumentsRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["export_documents"]
 
@@ -507,7 +506,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
     def import_documents(
         self,
     ) -> Callable[
-        [firestore_admin.ImportDocumentsRequest], Awaitable[operations.Operation]
+        [firestore_admin.ImportDocumentsRequest], Awaitable[operations_pb2.Operation]
     ]:
         r"""Return a callable for the import documents method over gRPC.
 
@@ -533,7 +532,7 @@ class FirestoreAdminGrpcAsyncIOTransport(FirestoreAdminTransport):
             self._stubs["import_documents"] = self.grpc_channel.unary_unary(
                 "/google.firestore.admin.v1.FirestoreAdmin/ImportDocuments",
                 request_serializer=firestore_admin.ImportDocumentsRequest.serialize,
-                response_deserializer=operations.Operation.FromString,
+                response_deserializer=operations_pb2.Operation.FromString,
             )
         return self._stubs["import_documents"]
 
