@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,10 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import proto  # type: ignore
 
-
 from google.cloud.firestore_admin_v1.types import index
 
 
@@ -65,7 +62,6 @@ class Field(proto.Message):
 
     class IndexConfig(proto.Message):
         r"""The index configuration for this field.
-
         Attributes:
             indexes (Sequence[google.cloud.firestore_admin_v1.types.Index]):
                 The indexes supported for this field.
@@ -90,15 +86,11 @@ class Field(proto.Message):
         """
 
         indexes = proto.RepeatedField(proto.MESSAGE, number=1, message=index.Index,)
+        uses_ancestor_config = proto.Field(proto.BOOL, number=2,)
+        ancestor_field = proto.Field(proto.STRING, number=3,)
+        reverting = proto.Field(proto.BOOL, number=4,)
 
-        uses_ancestor_config = proto.Field(proto.BOOL, number=2)
-
-        ancestor_field = proto.Field(proto.STRING, number=3)
-
-        reverting = proto.Field(proto.BOOL, number=4)
-
-    name = proto.Field(proto.STRING, number=1)
-
+    name = proto.Field(proto.STRING, number=1,)
     index_config = proto.Field(proto.MESSAGE, number=2, message=IndexConfig,)
 
 
