@@ -15,23 +15,40 @@
 """Common helpers shared across Google Cloud Firestore modules."""
 
 import datetime
+import json
 
+import google
+from google.api_core.datetime_helpers import DatetimeWithNanoseconds  # type: ignore
+from google.api_core import gapic_v1  # type: ignore
 from google.protobuf import struct_pb2
-from google.type import latlng_pb2
-import grpc
-import six
+from google.type import latlng_pb2  # type: ignore
+import grpc  # type: ignore
 
-from google.cloud import exceptions
-from google.cloud._helpers import _datetime_to_pb_timestamp
-from google.api_core.datetime_helpers import DatetimeWithNanoseconds
+from google.cloud import exceptions  # type: ignore
+from google.cloud._helpers import _datetime_to_pb_timestamp  # type: ignore
+from google.cloud.firestore_v1.types.write import DocumentTransform
 from google.cloud.firestore_v1 import transforms
 from google.cloud.firestore_v1 import types
 from google.cloud.firestore_v1.field_path import FieldPath
 from google.cloud.firestore_v1.field_path import parse_field_path
-from google.cloud.firestore_v1.gapic import enums
-from google.cloud.firestore_v1.proto import common_pb2
-from google.cloud.firestore_v1.proto import document_pb2
-from google.cloud.firestore_v1.proto import write_pb2
+from google.cloud.firestore_v1.types import common
+from google.cloud.firestore_v1.types import document
+from google.cloud.firestore_v1.types import write
+from google.protobuf.timestamp_pb2 import Timestamp  # type: ignore
+from typing import (
+    Any,
+    Dict,
+    Generator,
+    Iterator,
+    List,
+    NoReturn,
+    Optional,
+    Tuple,
+    Union,
+)
+
+_EmptyDict: transforms.Sentinel
+_GRPC_ERROR_MAPPING: dict
 
 
 BAD_PATH_TEMPLATE = "A path element must be a string. Received {}, which is a {}."
@@ -46,7 +63,7 @@ BAD_REFERENCE_ERROR = (
 WRONG_APP_REFERENCE = (
     "Document {!r} does not correspond to the same database " "({!r}) as the client."
 )
-REQUEST_TIME_ENUM = enums.DocumentTransform.FieldTransform.ServerValue.REQUEST_TIME
+REQUEST_TIME_ENUM = DocumentTransform.FieldTransform.ServerValue.REQUEST_TIME
 _GRPC_ERROR_MAPPING = {
     grpc.StatusCode.ALREADY_EXISTS: exceptions.Conflict,
     grpc.StatusCode.NOT_FOUND: exceptions.NotFound,
@@ -61,11 +78,11 @@ class GeoPoint(object):
         longitude (float): Longitude of a point.
     """
 
-    def __init__(self, latitude, longitude):
+    def __init__(self, latitude, longitude) -> None:
         self.latitude = latitude
         self.longitude = longitude
 
-    def to_protobuf(self):
+    def to_protobuf(self) -> latlng_pb2.LatLng:
         """Convert the current object to protobuf.
 
         Returns:
@@ -101,7 +118,7 @@ class GeoPoint(object):
             return not equality_val
 
 
-def verify_path(path, is_collection):
+def verify_path(path, is_collection) -> None:
     """Verifies that a ``path`` has the correct form.
 
     Checks that all of the elements in ``path`` are strings.
@@ -132,12 +149,12 @@ def verify_path(path, is_collection):
             raise ValueError("A document must have an even number of path elements")
 
     for element in path:
-        if not isinstance(element, six.string_types):
+        if not isinstance(element, str):
             msg = BAD_PATH_TEMPLATE.format(element, type(element))
             raise ValueError(msg)
 
 
-def encode_value(value):
+def encode_value(value) -> types.document.Value:
     """Converts a native Python value into a Firestore protobuf ``Value``.
 
     Args:
@@ -153,55 +170,55 @@ def encode_value(value):
         TypeError: If the ``value`` is not one of the accepted types.
     """
     if value is None:
-        return document_pb2.Value(null_value=struct_pb2.NULL_VALUE)
+        return document.Value(null_value=struct_pb2.NULL_VALUE)
 
-    # Must come before six.integer_types since ``bool`` is an integer subtype.
+    # Must come before int since ``bool`` is an integer subtype.
     if isinstance(value, bool):
-        return document_pb2.Value(boolean_value=value)
+        return document.Value(boolean_value=value)
 
-    if isinstance(value, six.integer_types):
-        return document_pb2.Value(integer_value=value)
+    if isinstance(value, int):
+        return document.Value(integer_value=value)
 
     if isinstance(value, float):
-        return document_pb2.Value(double_value=value)
+        return document.Value(double_value=value)
 
     if isinstance(value, DatetimeWithNanoseconds):
-        return document_pb2.Value(timestamp_value=value.timestamp_pb())
+        return document.Value(timestamp_value=value.timestamp_pb())
 
     if isinstance(value, datetime.datetime):
-        return document_pb2.Value(timestamp_value=_datetime_to_pb_timestamp(value))
+        return document.Value(timestamp_value=_datetime_to_pb_timestamp(value))
 
-    if isinstance(value, six.text_type):
-        return document_pb2.Value(string_value=value)
+    if isinstance(value, str):
+        return document.Value(string_value=value)
 
-    if isinstance(value, six.binary_type):
-        return document_pb2.Value(bytes_value=value)
+    if isinstance(value, bytes):
+        return document.Value(bytes_value=value)
 
     # NOTE: We avoid doing an isinstance() check for a Document
     #       here to avoid import cycles.
     document_path = getattr(value, "_document_path", None)
     if document_path is not None:
-        return document_pb2.Value(reference_value=document_path)
+        return document.Value(reference_value=document_path)
 
     if isinstance(value, GeoPoint):
-        return document_pb2.Value(geo_point_value=value.to_protobuf())
+        return document.Value(geo_point_value=value.to_protobuf())
 
     if isinstance(value, (list, tuple, set, frozenset)):
         value_list = tuple(encode_value(element) for element in value)
-        value_pb = document_pb2.ArrayValue(values=value_list)
-        return document_pb2.Value(array_value=value_pb)
+        value_pb = document.ArrayValue(values=value_list)
+        return document.Value(array_value=value_pb)
 
     if isinstance(value, dict):
         value_dict = encode_dict(value)
-        value_pb = document_pb2.MapValue(fields=value_dict)
-        return document_pb2.Value(map_value=value_pb)
+        value_pb = document.MapValue(fields=value_dict)
+        return document.Value(map_value=value_pb)
 
     raise TypeError(
         "Cannot convert to a Firestore Value", value, "Invalid type", type(value)
     )
 
 
-def encode_dict(values_dict):
+def encode_dict(values_dict) -> dict:
     """Encode a dictionary into protobuf ``Value``-s.
 
     Args:
@@ -212,10 +229,76 @@ def encode_dict(values_dict):
         dictionary of string keys and ``Value`` protobufs as dictionary
         values.
     """
-    return {key: encode_value(value) for key, value in six.iteritems(values_dict)}
+    return {key: encode_value(value) for key, value in values_dict.items()}
 
 
-def reference_value_to_document(reference_value, client):
+def document_snapshot_to_protobuf(snapshot: "google.cloud.firestore_v1.base_document.DocumentSnapshot") -> Optional["google.cloud.firestore_v1.types.Document"]:  # type: ignore
+    from google.cloud.firestore_v1.types import Document
+
+    if not snapshot.exists:
+        return None
+
+    return Document(
+        name=snapshot.reference._document_path,
+        fields=encode_dict(snapshot._data),
+        create_time=snapshot.create_time,
+        update_time=snapshot.update_time,
+    )
+
+
+class DocumentReferenceValue:
+    """DocumentReference path container with accessors for each relevant chunk.
+
+    Usage:
+        doc_ref_val = DocumentReferenceValue(
+            'projects/my-proj/databases/(default)/documents/my-col/my-doc',
+        )
+        assert doc_ref_val.project_name == 'my-proj'
+        assert doc_ref_val.collection_name == 'my-col'
+        assert doc_ref_val.document_id == 'my-doc'
+        assert doc_ref_val.database_name == '(default)'
+
+    Raises:
+        ValueError: If the supplied value cannot satisfy a complete path.
+    """
+
+    def __init__(self, reference_value: str):
+        self._reference_value = reference_value
+
+        # The first 5 parts are
+        # projects, {project}, databases, {database}, documents
+        parts = reference_value.split(DOCUMENT_PATH_DELIMITER)
+        if len(parts) < 7:
+            msg = BAD_REFERENCE_ERROR.format(reference_value)
+            raise ValueError(msg)
+
+        self.project_name = parts[1]
+        self.collection_name = parts[5]
+        self.database_name = parts[3]
+        self.document_id = "/".join(parts[6:])
+
+    @property
+    def full_key(self) -> str:
+        """Computed property for a DocumentReference's collection_name and
+        document Id"""
+        return "/".join([self.collection_name, self.document_id])
+
+    @property
+    def full_path(self) -> str:
+        return self._reference_value or "/".join(
+            [
+                "projects",
+                self.project_name,
+                "databases",
+                self.database_name,
+                "documents",
+                self.collection_name,
+                self.document_id,
+            ]
+        )
+
+
+def reference_value_to_document(reference_value, client) -> Any:
     """Convert a reference value string to a document.
 
     Args:
@@ -233,15 +316,11 @@ def reference_value_to_document(reference_value, client):
         ValueError: If the ``reference_value`` does not come from the same
             project / database combination as the ``client``.
     """
-    # The first 5 parts are
-    # projects, {project}, databases, {database}, documents
-    parts = reference_value.split(DOCUMENT_PATH_DELIMITER, 5)
-    if len(parts) != 6:
-        msg = BAD_REFERENCE_ERROR.format(reference_value)
-        raise ValueError(msg)
+    from google.cloud.firestore_v1.base_document import BaseDocumentReference
 
-    # The sixth part is `a/b/c/d` (i.e. the document path)
-    document = client.document(parts[-1])
+    doc_ref_value = DocumentReferenceValue(reference_value)
+
+    document: BaseDocumentReference = client.document(doc_ref_value.full_key)
     if document._document_path != reference_value:
         msg = WRONG_APP_REFERENCE.format(reference_value, client._database_string)
         raise ValueError(msg)
@@ -249,7 +328,9 @@ def reference_value_to_document(reference_value, client):
     return document
 
 
-def decode_value(value, client):
+def decode_value(
+    value, client
+) -> Union[None, bool, int, float, list, datetime.datetime, str, bytes, dict, GeoPoint]:
     """Converts a Firestore protobuf ``Value`` to a native Python value.
 
     Args:
@@ -267,7 +348,7 @@ def decode_value(value, client):
         NotImplementedError: If the ``value_type`` is ``reference_value``.
         ValueError: If the ``value_type`` is unknown.
     """
-    value_type = value.WhichOneof("value_type")
+    value_type = value._pb.WhichOneof("value_type")
 
     if value_type == "null_value":
         return None
@@ -278,7 +359,7 @@ def decode_value(value, client):
     elif value_type == "double_value":
         return value.double_value
     elif value_type == "timestamp_value":
-        return DatetimeWithNanoseconds.from_timestamp_pb(value.timestamp_value)
+        return DatetimeWithNanoseconds.from_timestamp_pb(value._pb.timestamp_value)
     elif value_type == "string_value":
         return value.string_value
     elif value_type == "bytes_value":
@@ -295,7 +376,7 @@ def decode_value(value, client):
         raise ValueError("Unknown ``value_type``", value_type)
 
 
-def decode_dict(value_fields, client):
+def decode_dict(value_fields, client) -> dict:
     """Converts a protobuf map of Firestore ``Value``-s.
 
     Args:
@@ -309,17 +390,15 @@ def decode_dict(value_fields, client):
             str, bytes, dict, ~google.cloud.Firestore.GeoPoint]]: A dictionary
         of native Python values converted from the ``value_fields``.
     """
-    return {
-        key: decode_value(value, client) for key, value in six.iteritems(value_fields)
-    }
+    return {key: decode_value(value, client) for key, value in value_fields.items()}
 
 
-def get_doc_id(document_pb, expected_prefix):
+def get_doc_id(document_pb, expected_prefix) -> str:
     """Parse a document ID from a document protobuf.
 
     Args:
         document_pb (google.cloud.proto.firestore.v1.\
-            document_pb2.Document): A protobuf for a document that
+            document.Document): A protobuf for a document that
             was created in a ``CreateDocument`` RPC.
         expected_prefix (str): The expected collection prefix for the
             fully-qualified document name.
@@ -345,12 +424,14 @@ def get_doc_id(document_pb, expected_prefix):
 _EmptyDict = transforms.Sentinel("Marker for an empty dict value")
 
 
-def extract_fields(document_data, prefix_path, expand_dots=False):
+def extract_fields(
+    document_data, prefix_path: FieldPath, expand_dots=False
+) -> Generator[Tuple[Any, Any], Any, None]:
     """Do depth-first walk of tree, yielding field_path, value"""
     if not document_data:
         yield prefix_path, _EmptyDict
     else:
-        for key, value in sorted(six.iteritems(document_data)):
+        for key, value in sorted(document_data.items()):
 
             if expand_dots:
                 sub_key = FieldPath.from_string(key)
@@ -366,7 +447,7 @@ def extract_fields(document_data, prefix_path, expand_dots=False):
                 yield field_path, value
 
 
-def set_field_value(document_data, field_path, value):
+def set_field_value(document_data, field_path, value) -> None:
     """Set a value into a document for a field_path"""
     current = document_data
     for element in field_path.parts[:-1]:
@@ -376,7 +457,7 @@ def set_field_value(document_data, field_path, value):
     current[field_path.parts[-1]] = value
 
 
-def get_field_value(document_data, field_path):
+def get_field_value(document_data, field_path) -> Any:
     if not field_path.parts:
         raise ValueError("Empty path")
 
@@ -397,7 +478,7 @@ class DocumentExtractor(object):
             a document.
     """
 
-    def __init__(self, document_data):
+    def __init__(self, document_data) -> None:
         self.document_data = document_data
         self.field_paths = []
         self.deleted_fields = []
@@ -443,7 +524,9 @@ class DocumentExtractor(object):
                 self.field_paths.append(field_path)
                 set_field_value(self.set_fields, field_path, value)
 
-    def _get_document_iterator(self, prefix_path):
+    def _get_document_iterator(
+        self, prefix_path: FieldPath
+    ) -> Generator[Tuple[Any, Any], Any, None]:
         return extract_fields(self.document_data, prefix_path)
 
     @property
@@ -468,18 +551,20 @@ class DocumentExtractor(object):
             + list(self.minimums)
         )
 
-    def _get_update_mask(self, allow_empty_mask=False):
+    def _get_update_mask(self, allow_empty_mask=False) -> None:
         return None
 
-    def get_update_pb(self, document_path, exists=None, allow_empty_mask=False):
+    def get_update_pb(
+        self, document_path, exists=None, allow_empty_mask=False
+    ) -> types.write.Write:
 
         if exists is not None:
-            current_document = common_pb2.Precondition(exists=exists)
+            current_document = common.Precondition(exists=exists)
         else:
             current_document = None
 
-        update_pb = write_pb2.Write(
-            update=document_pb2.Document(
+        update_pb = write.Write(
+            update=document.Document(
                 name=document_path, fields=encode_dict(self.set_fields)
             ),
             update_mask=self._get_update_mask(allow_empty_mask),
@@ -488,16 +573,18 @@ class DocumentExtractor(object):
 
         return update_pb
 
-    def get_transform_pb(self, document_path, exists=None):
+    def get_field_transform_pbs(
+        self, document_path
+    ) -> List[types.write.DocumentTransform.FieldTransform]:
         def make_array_value(values):
             value_list = [encode_value(element) for element in values]
-            return document_pb2.ArrayValue(values=value_list)
+            return document.ArrayValue(values=value_list)
 
         path_field_transforms = (
             [
                 (
                     path,
-                    write_pb2.DocumentTransform.FieldTransform(
+                    write.DocumentTransform.FieldTransform(
                         field_path=path.to_api_repr(),
                         set_to_server_value=REQUEST_TIME_ENUM,
                     ),
@@ -507,7 +594,7 @@ class DocumentExtractor(object):
             + [
                 (
                     path,
-                    write_pb2.DocumentTransform.FieldTransform(
+                    write.DocumentTransform.FieldTransform(
                         field_path=path.to_api_repr(),
                         remove_all_from_array=make_array_value(values),
                     ),
@@ -517,7 +604,7 @@ class DocumentExtractor(object):
             + [
                 (
                     path,
-                    write_pb2.DocumentTransform.FieldTransform(
+                    write.DocumentTransform.FieldTransform(
                         field_path=path.to_api_repr(),
                         append_missing_elements=make_array_value(values),
                     ),
@@ -527,7 +614,7 @@ class DocumentExtractor(object):
             + [
                 (
                     path,
-                    write_pb2.DocumentTransform.FieldTransform(
+                    write.DocumentTransform.FieldTransform(
                         field_path=path.to_api_repr(), increment=encode_value(value)
                     ),
                 )
@@ -536,7 +623,7 @@ class DocumentExtractor(object):
             + [
                 (
                     path,
-                    write_pb2.DocumentTransform.FieldTransform(
+                    write.DocumentTransform.FieldTransform(
                         field_path=path.to_api_repr(), maximum=encode_value(value)
                     ),
                 )
@@ -545,30 +632,31 @@ class DocumentExtractor(object):
             + [
                 (
                     path,
-                    write_pb2.DocumentTransform.FieldTransform(
+                    write.DocumentTransform.FieldTransform(
                         field_path=path.to_api_repr(), minimum=encode_value(value)
                     ),
                 )
                 for path, value in self.minimums.items()
             ]
         )
-        field_transforms = [
-            transform for path, transform in sorted(path_field_transforms)
-        ]
-        transform_pb = write_pb2.Write(
-            transform=write_pb2.DocumentTransform(
+        return [transform for path, transform in sorted(path_field_transforms)]
+
+    def get_transform_pb(self, document_path, exists=None) -> types.write.Write:
+        field_transforms = self.get_field_transform_pbs(document_path)
+        transform_pb = write.Write(
+            transform=write.DocumentTransform(
                 document=document_path, field_transforms=field_transforms
             )
         )
         if exists is not None:
-            transform_pb.current_document.CopyFrom(
-                common_pb2.Precondition(exists=exists)
+            transform_pb._pb.current_document.CopyFrom(
+                common.Precondition(exists=exists)._pb
             )
 
         return transform_pb
 
 
-def pbs_for_create(document_path, document_data):
+def pbs_for_create(document_path, document_data) -> List[types.write.Write]:
     """Make ``Write`` protobufs for ``create()`` methods.
 
     Args:
@@ -585,22 +673,16 @@ def pbs_for_create(document_path, document_data):
     if extractor.deleted_fields:
         raise ValueError("Cannot apply DELETE_FIELD in a create request.")
 
-    write_pbs = []
-
-    # Conformance tests require skipping the 'update_pb' if the document
-    # contains only transforms.
-    if extractor.empty_document or extractor.set_fields:
-        write_pbs.append(extractor.get_update_pb(document_path, exists=False))
+    create_pb = extractor.get_update_pb(document_path, exists=False)
 
     if extractor.has_transforms:
-        exists = None if write_pbs else False
-        transform_pb = extractor.get_transform_pb(document_path, exists)
-        write_pbs.append(transform_pb)
+        field_transform_pbs = extractor.get_field_transform_pbs(document_path)
+        create_pb.update_transforms.extend(field_transform_pbs)
 
-    return write_pbs
+    return [create_pb]
 
 
-def pbs_for_set_no_merge(document_path, document_data):
+def pbs_for_set_no_merge(document_path, document_data) -> List[types.write.Write]:
     """Make ``Write`` protobufs for ``set()`` methods.
 
     Args:
@@ -620,55 +702,39 @@ def pbs_for_set_no_merge(document_path, document_data):
             "specifying 'merge=True' or 'merge=[field_paths]'."
         )
 
-    # Conformance tests require send the 'update_pb' even if the document
-    # contains only transforms.
-    write_pbs = [extractor.get_update_pb(document_path)]
+    set_pb = extractor.get_update_pb(document_path)
 
     if extractor.has_transforms:
-        transform_pb = extractor.get_transform_pb(document_path)
-        write_pbs.append(transform_pb)
+        field_transform_pbs = extractor.get_field_transform_pbs(document_path)
+        set_pb.update_transforms.extend(field_transform_pbs)
 
-    return write_pbs
+    return [set_pb]
 
 
 class DocumentExtractorForMerge(DocumentExtractor):
     """ Break document data up into actual data and transforms.
     """
 
-    def __init__(self, document_data):
+    def __init__(self, document_data) -> None:
         super(DocumentExtractorForMerge, self).__init__(document_data)
         self.data_merge = []
         self.transform_merge = []
         self.merge = []
 
-    @property
-    def has_updates(self):
-        # for whatever reason, the conformance tests want to see the parent
-        # of nested transform paths in the update mask
-        # (see set-st-merge-nonleaf-alone.textproto)
-        update_paths = set(self.data_merge)
-
-        for transform_path in self.transform_paths:
-            if len(transform_path.parts) > 1:
-                parent_fp = FieldPath(*transform_path.parts[:-1])
-                update_paths.add(parent_fp)
-
-        return bool(update_paths)
-
-    def _apply_merge_all(self):
+    def _apply_merge_all(self) -> None:
         self.data_merge = sorted(self.field_paths + self.deleted_fields)
         # TODO: other transforms
         self.transform_merge = self.transform_paths
         self.merge = sorted(self.data_merge + self.transform_paths)
 
-    def _construct_merge_paths(self, merge):
+    def _construct_merge_paths(self, merge) -> Generator[Any, Any, None]:
         for merge_field in merge:
             if isinstance(merge_field, FieldPath):
                 yield merge_field
             else:
                 yield FieldPath(*parse_field_path(merge_field))
 
-    def _normalize_merge_paths(self, merge):
+    def _normalize_merge_paths(self, merge) -> list:
         merge_paths = sorted(self._construct_merge_paths(merge))
 
         # Raise if any merge path is a parent of another.  Leverage sorting
@@ -688,7 +754,7 @@ class DocumentExtractorForMerge(DocumentExtractor):
 
         return merge_paths
 
-    def _apply_merge_paths(self, merge):
+    def _apply_merge_paths(self, merge) -> None:
 
         if self.empty_document:
             raise ValueError("Cannot merge specific fields with empty document.")
@@ -752,13 +818,15 @@ class DocumentExtractorForMerge(DocumentExtractor):
             if path in merged_transform_paths
         }
 
-    def apply_merge(self, merge):
+    def apply_merge(self, merge) -> None:
         if merge is True:  # merge all fields
             self._apply_merge_all()
         else:
             self._apply_merge_paths(merge)
 
-    def _get_update_mask(self, allow_empty_mask=False):
+    def _get_update_mask(
+        self, allow_empty_mask=False
+    ) -> Optional[types.common.DocumentMask]:
         # Mask uses dotted / quoted paths.
         mask_paths = [
             field_path.to_api_repr()
@@ -766,11 +834,12 @@ class DocumentExtractorForMerge(DocumentExtractor):
             if field_path not in self.transform_merge
         ]
 
-        if mask_paths or allow_empty_mask:
-            return common_pb2.DocumentMask(field_paths=mask_paths)
+        return common.DocumentMask(field_paths=mask_paths)
 
 
-def pbs_for_set_with_merge(document_path, document_data, merge):
+def pbs_for_set_with_merge(
+    document_path, document_data, merge
+) -> List[types.write.Write]:
     """Make ``Write`` protobufs for ``set()`` methods.
 
     Args:
@@ -787,27 +856,20 @@ def pbs_for_set_with_merge(document_path, document_data, merge):
     extractor = DocumentExtractorForMerge(document_data)
     extractor.apply_merge(merge)
 
-    merge_empty = not document_data
-
-    write_pbs = []
-
-    if extractor.has_updates or merge_empty:
-        write_pbs.append(
-            extractor.get_update_pb(document_path, allow_empty_mask=merge_empty)
-        )
+    set_pb = extractor.get_update_pb(document_path)
 
     if extractor.transform_paths:
-        transform_pb = extractor.get_transform_pb(document_path)
-        write_pbs.append(transform_pb)
+        field_transform_pbs = extractor.get_field_transform_pbs(document_path)
+        set_pb.update_transforms.extend(field_transform_pbs)
 
-    return write_pbs
+    return [set_pb]
 
 
 class DocumentExtractorForUpdate(DocumentExtractor):
     """ Break document data up into actual data and transforms.
     """
 
-    def __init__(self, document_data):
+    def __init__(self, document_data) -> None:
         super(DocumentExtractorForUpdate, self).__init__(document_data)
         self.top_level_paths = sorted(
             [FieldPath.from_string(key) for key in document_data]
@@ -828,19 +890,21 @@ class DocumentExtractorForUpdate(DocumentExtractor):
                     "Cannot update with nest delete: {}".format(field_path)
                 )
 
-    def _get_document_iterator(self, prefix_path):
+    def _get_document_iterator(
+        self, prefix_path: FieldPath
+    ) -> Generator[Tuple[Any, Any], Any, None]:
         return extract_fields(self.document_data, prefix_path, expand_dots=True)
 
-    def _get_update_mask(self, allow_empty_mask=False):
+    def _get_update_mask(self, allow_empty_mask=False) -> types.common.DocumentMask:
         mask_paths = []
         for field_path in self.top_level_paths:
             if field_path not in self.transform_paths:
                 mask_paths.append(field_path.to_api_repr())
 
-        return common_pb2.DocumentMask(field_paths=mask_paths)
+        return common.DocumentMask(field_paths=mask_paths)
 
 
-def pbs_for_update(document_path, field_updates, option):
+def pbs_for_update(document_path, field_updates, option) -> List[types.write.Write]:
     """Make ``Write`` protobufs for ``update()`` methods.
 
     Args:
@@ -863,25 +927,17 @@ def pbs_for_update(document_path, field_updates, option):
     if option is None:  # Default is to use ``exists=True``.
         option = ExistsOption(exists=True)
 
-    write_pbs = []
-
-    if extractor.field_paths or extractor.deleted_fields:
-        update_pb = extractor.get_update_pb(document_path)
-        option.modify_write(update_pb)
-        write_pbs.append(update_pb)
+    update_pb = extractor.get_update_pb(document_path)
+    option.modify_write(update_pb)
 
     if extractor.has_transforms:
-        transform_pb = extractor.get_transform_pb(document_path)
-        if not write_pbs:
-            # NOTE: set the write option on the ``transform_pb`` only if there
-            #       is no ``update_pb``
-            option.modify_write(transform_pb)
-        write_pbs.append(transform_pb)
+        field_transform_pbs = extractor.get_field_transform_pbs(document_path)
+        update_pb.update_transforms.extend(field_transform_pbs)
 
-    return write_pbs
+    return [update_pb]
 
 
-def pb_for_delete(document_path, option):
+def pb_for_delete(document_path, option) -> types.write.Write:
     """Make a ``Write`` protobuf for ``delete()`` methods.
 
     Args:
@@ -894,7 +950,7 @@ def pb_for_delete(document_path, option):
         google.cloud.firestore_v1.types.Write: A
         ``Write`` protobuf instance for the ``delete()``.
     """
-    write_pb = write_pb2.Write(delete=document_path)
+    write_pb = write.Write(delete=document_path)
     if option is not None:
         option.modify_write(write_pb)
 
@@ -908,7 +964,7 @@ class ReadAfterWriteError(Exception):
     """
 
 
-def get_transaction_id(transaction, read_operation=True):
+def get_transaction_id(transaction, read_operation=True) -> Union[bytes, None]:
     """Get the transaction ID from a ``Transaction`` object.
 
     Args:
@@ -938,7 +994,7 @@ def get_transaction_id(transaction, read_operation=True):
         return transaction.id
 
 
-def metadata_with_prefix(prefix, **kw):
+def metadata_with_prefix(prefix: str, **kw) -> List[Tuple[str, str]]:
     """Create RPC metadata containing a prefix.
 
     Args:
@@ -953,13 +1009,13 @@ def metadata_with_prefix(prefix, **kw):
 class WriteOption(object):
     """Option used to assert a condition on a write operation."""
 
-    def modify_write(self, write_pb, no_create_msg=None):
+    def modify_write(self, write, no_create_msg=None) -> NoReturn:
         """Modify a ``Write`` protobuf based on the state of this write option.
 
         This is a virtual method intended to be implemented by subclasses.
 
         Args:
-            write_pb (google.cloud.firestore_v1.types.Write): A
+            write (google.cloud.firestore_v1.types.Write): A
                 ``Write`` protobuf instance to be modified with a precondition
                 determined by the state of this option.
             no_create_msg (Optional[str]): A message to use to indicate that
@@ -985,7 +1041,7 @@ class LastUpdateOption(WriteOption):
             as part of a "write result" protobuf or directly.
     """
 
-    def __init__(self, last_update_time):
+    def __init__(self, last_update_time) -> None:
         self._last_update_time = last_update_time
 
     def __eq__(self, other):
@@ -993,7 +1049,7 @@ class LastUpdateOption(WriteOption):
             return NotImplemented
         return self._last_update_time == other._last_update_time
 
-    def modify_write(self, write_pb, **unused_kwargs):
+    def modify_write(self, write, **unused_kwargs) -> None:
         """Modify a ``Write`` protobuf based on the state of this write option.
 
         The ``last_update_time`` is added to ``write_pb`` as an "update time"
@@ -1008,7 +1064,7 @@ class LastUpdateOption(WriteOption):
                 other subclasses that are unused here.
         """
         current_doc = types.Precondition(update_time=self._last_update_time)
-        write_pb.current_document.CopyFrom(current_doc)
+        write._pb.current_document.CopyFrom(current_doc._pb)
 
 
 class ExistsOption(WriteOption):
@@ -1022,7 +1078,7 @@ class ExistsOption(WriteOption):
             should already exist.
     """
 
-    def __init__(self, exists):
+    def __init__(self, exists) -> None:
         self._exists = exists
 
     def __eq__(self, other):
@@ -1030,7 +1086,7 @@ class ExistsOption(WriteOption):
             return NotImplemented
         return self._exists == other._exists
 
-    def modify_write(self, write_pb, **unused_kwargs):
+    def modify_write(self, write, **unused_kwargs) -> None:
         """Modify a ``Write`` protobuf based on the state of this write option.
 
         If:
@@ -1039,11 +1095,200 @@ class ExistsOption(WriteOption):
         * ``exists=False``, adds a precondition that requires non-existence
 
         Args:
-            write_pb (google.cloud.firestore_v1.types.Write): A
+            write (google.cloud.firestore_v1.types.Write): A
                 ``Write`` protobuf instance to be modified with a precondition
                 determined by the state of this option.
             unused_kwargs (Dict[str, Any]): Keyword arguments accepted by
                 other subclasses that are unused here.
         """
         current_doc = types.Precondition(exists=self._exists)
-        write_pb.current_document.CopyFrom(current_doc)
+        write._pb.current_document.CopyFrom(current_doc._pb)
+
+
+def make_retry_timeout_kwargs(retry, timeout) -> dict:
+    """Helper fo API methods which take optional 'retry' / 'timeout' args."""
+    kwargs = {}
+
+    if retry is not gapic_v1.method.DEFAULT:
+        kwargs["retry"] = retry
+
+    if timeout is not None:
+        kwargs["timeout"] = timeout
+
+    return kwargs
+
+
+def build_timestamp(
+    dt: Optional[Union[DatetimeWithNanoseconds, datetime.datetime]] = None
+) -> Timestamp:
+    """Returns the supplied datetime (or "now") as a Timestamp"""
+    return _datetime_to_pb_timestamp(dt or DatetimeWithNanoseconds.utcnow())
+
+
+def compare_timestamps(
+    ts1: Union[Timestamp, datetime.datetime], ts2: Union[Timestamp, datetime.datetime],
+) -> int:
+    ts1 = build_timestamp(ts1) if not isinstance(ts1, Timestamp) else ts1
+    ts2 = build_timestamp(ts2) if not isinstance(ts2, Timestamp) else ts2
+    ts1_nanos = ts1.nanos + ts1.seconds * 1e9
+    ts2_nanos = ts2.nanos + ts2.seconds * 1e9
+    if ts1_nanos == ts2_nanos:
+        return 0
+    return 1 if ts1_nanos > ts2_nanos else -1
+
+
+def deserialize_bundle(
+    serialized: Union[str, bytes],
+    client: "google.cloud.firestore_v1.client.BaseClient",  # type: ignore
+) -> "google.cloud.firestore_bundle.FirestoreBundle":  # type: ignore
+    """Inverse operation to a `FirestoreBundle` instance's `build()` method.
+
+    Args:
+        serialized (Union[str, bytes]): The result of `FirestoreBundle.build()`.
+            Should be a list of dictionaries in string format.
+        client (BaseClient): A connected Client instance.
+
+    Returns:
+        FirestoreBundle: A bundle equivalent to that which called `build()` and
+            initially created the `serialized` value.
+
+    Raises:
+        ValueError: If any of the dictionaries in the list contain any more than
+            one top-level key.
+        ValueError: If any unexpected BundleElement types are encountered.
+        ValueError: If the serialized bundle ends before expected.
+    """
+    from google.cloud.firestore_bundle import BundleElement, FirestoreBundle
+
+    # Outlines the legal transitions from one BundleElement to another.
+    bundle_state_machine = {
+        "__initial__": ["metadata"],
+        "metadata": ["namedQuery", "documentMetadata", "__end__"],
+        "namedQuery": ["namedQuery", "documentMetadata", "__end__"],
+        "documentMetadata": ["document"],
+        "document": ["documentMetadata", "__end__"],
+    }
+    allowed_next_element_types: List[str] = bundle_state_machine["__initial__"]
+
+    # This must be saved and added last, since we cache it to preserve timestamps,
+    # yet must flush it whenever a new document or query is added to a bundle.
+    # The process of deserializing a bundle uses these methods which flush a
+    # cached metadata element, and thus, it must be the last BundleElement
+    # added during deserialization.
+    metadata_bundle_element: Optional[BundleElement] = None
+
+    bundle: Optional[FirestoreBundle] = None
+    data: Dict
+    for data in _parse_bundle_elements_data(serialized):
+
+        # BundleElements are serialized as JSON containing one key outlining
+        # the type, with all further data nested under that key
+        keys: List[str] = list(data.keys())
+
+        if len(keys) != 1:
+            raise ValueError("Expected serialized BundleElement with one top-level key")
+
+        key: str = keys[0]
+
+        if key not in allowed_next_element_types:
+            raise ValueError(
+                f"Encountered BundleElement of type {key}. "
+                f"Expected one of {allowed_next_element_types}"
+            )
+
+        # Create and add our BundleElement
+        bundle_element: BundleElement
+        try:
+            bundle_element: BundleElement = BundleElement.from_json(json.dumps(data))  # type: ignore
+        except AttributeError as e:
+            # Some bad serialization formats cannot be universally deserialized.
+            if e.args[0] == "'dict' object has no attribute 'find'":
+                raise ValueError(
+                    "Invalid serialization of datetimes. "
+                    "Cannot deserialize Bundles created from the NodeJS SDK."
+                )
+            raise e  # pragma: NO COVER
+
+        if bundle is None:
+            # This must be the first bundle type encountered
+            assert key == "metadata"
+            bundle = FirestoreBundle(data[key]["id"])
+            metadata_bundle_element = bundle_element
+
+        else:
+            bundle._add_bundle_element(bundle_element, client=client, type=key)
+
+        # Update the allowed next BundleElement types
+        allowed_next_element_types = bundle_state_machine[key]
+
+    if "__end__" not in allowed_next_element_types:
+        raise ValueError("Unexpected end to serialized FirestoreBundle")
+
+    # Now, finally add the metadata element
+    bundle._add_bundle_element(
+        metadata_bundle_element, client=client, type="metadata",  # type: ignore
+    )
+
+    return bundle
+
+
+def _parse_bundle_elements_data(serialized: Union[str, bytes]) -> Generator[Dict, None, None]:  # type: ignore
+    """Reads through a serialized FirestoreBundle and yields JSON chunks that
+    were created via `BundleElement.to_json(bundle_element)`.
+
+    Serialized FirestoreBundle instances are length-prefixed JSON objects, and
+    so are of the form "123{...}57{...}"
+    To correctly and safely read a bundle, we must first detect these length
+    prefixes, read that many bytes of data, and attempt to JSON-parse that.
+
+    Raises:
+        ValueError: If a chunk of JSON ever starts without following a length
+            prefix.
+    """
+    _serialized: Iterator[int] = iter(
+        serialized if isinstance(serialized, bytes) else serialized.encode("utf-8")
+    )
+
+    length_prefix: str = ""
+    while True:
+        byte: Optional[int] = next(_serialized, None)
+
+        if byte is None:
+            return None
+
+        _str: str = chr(byte)
+        if _str.isnumeric():
+            length_prefix += _str
+        else:
+            if length_prefix == "":
+                raise ValueError("Expected length prefix")
+
+            _length_prefix = int(length_prefix)
+            length_prefix = ""
+            _bytes = bytearray([byte])
+            _counter = 1
+            while _counter < _length_prefix:
+                _bytes.append(next(_serialized))
+                _counter += 1
+
+            yield json.loads(_bytes.decode("utf-8"))
+
+
+def _get_documents_from_bundle(
+    bundle, *, query_name: Optional[str] = None
+) -> Generator["google.cloud.firestore.DocumentSnapshot", None, None]:  # type: ignore
+    from google.cloud.firestore_bundle.bundle import _BundledDocument
+
+    bundled_doc: _BundledDocument
+    for bundled_doc in bundle.documents.values():
+        if query_name and query_name not in bundled_doc.metadata.queries:
+            continue
+        yield bundled_doc.snapshot
+
+
+def _get_document_from_bundle(
+    bundle, *, document_id: str,
+) -> Optional["google.cloud.firestore.DocumentSnapshot"]:  # type: ignore
+    bundled_doc = bundle.documents.get(document_id)
+    if bundled_doc:
+        return bundled_doc.snapshot
