@@ -183,6 +183,10 @@ class BaseCollectionReference(object):
             "collection_id": self.id,
             "page_size": page_size,
             "show_missing": True,
+            # list_documents returns an iterator of document references, which do not
+            # include any fields. To save on data transfer, we can set a field_path mask
+            # to include no fields
+            "mask": {"field_paths": None},
         }
         kwargs = _helpers.make_retry_timeout_kwargs(retry, timeout)
 
@@ -284,12 +288,15 @@ class BaseCollectionReference(object):
 
     def limit_to_last(self, count: int):
         """Create a limited to last query with this collection as parent.
+
         .. note::
            `limit` and `limit_to_last` are mutually exclusive.
            Setting `limit_to_last` will drop previously set `limit`.
+
         See
         :meth:`~google.cloud.firestore_v1.query.Query.limit_to_last`
         for more information on this method.
+
         Args:
             count (int): Maximum number of documents to return that
                 match the query.
