@@ -13,24 +13,28 @@
 # limitations under the License.
 
 """Classes for representing documents for the Google Cloud Firestore API."""
+import datetime
+import logging
 
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
+from google.cloud._helpers import _datetime_to_pb_timestamp  # type: ignore
 
 from google.cloud.firestore_v1.base_document import (
     BaseDocumentReference,
     DocumentSnapshot,
     _first_write_result,
 )
-
-from google.api_core import exceptions  # type: ignore
 from google.cloud.firestore_v1 import _helpers
 from google.cloud.firestore_v1.types import write
 from google.cloud.firestore_v1.watch import Watch
-from google.protobuf import timestamp_pb2
+from google.protobuf.timestamp_pb2 import Timestamp
 from typing import Any, Callable, Generator, Iterable
 
 
+logger = logging.getLogger(__name__)
+
+
 class DocumentReference(BaseDocumentReference):
     """A reference to a document in a Firestore database.
 
@@ -65,7 +69,14 @@ class DocumentReference(BaseDocumentReference):
         retry: retries.Retry = gapic_v1.method.DEFAULT,
         timeout: float = None,
     ) -> write.WriteResult:
-        """Create the current document in the Firestore database.
+        """Create a document in the Firestore database.
+
+        >>> document_data = {"a": 1, "b": {"c": "Two"}}
+        >>> document.get().to_dict() is None  # does not exist
+        True
+        >>> document.create(document_data)
+        >>> document.get().to_dict() == document_data  # exists
+        True
 
         Args:
             document_data (dict): Property names and values to use for
@@ -95,23 +106,51 @@ class DocumentReference(BaseDocumentReference):
         retry: retries.Retry = gapic_v1.method.DEFAULT,
         timeout: float = None,
     ) -> write.WriteResult:
-        """Replace the current document in the Firestore database.
+        """Create / replace / merge a document in the Firestore database.
 
-        A write ``option`` can be specified to indicate preconditions of
-        the "set" operation. If no ``option`` is specified and this document
-        doesn't exist yet, this method will create it.
+        - To "upsert" a document (create if it doesn't exist, replace completely
+          if it does), leave the ``merge`` argument at its default:
 
-        Overwrites all content for the document with the fields in
-        ``document_data``. This method performs almost the same functionality
-        as :meth:`create`. The only difference is that this method doesn't
-        make any requirements on the existence of the document (unless
-        ``option`` is used), whereas as :meth:`create` will fail if the
-        document already exists.
+          >>> document_data = {"a": 1, "b": {"c": "Two"}}
+          >>> document.get().to_dict() is None  # document exists
+          False
+          >>> document.set(document_data)
+          >>> document.get().to_dict() == document_data  # exists
+          True
+
+        - To "merge" ``document_data`` with an existing document (creating if
+          the document does not exist), pass ``merge`` as True``:
+
+          >>> document_data = {"a": 1, "b": {"c": "Two"}}
+          >>> document.get().to_dict() == {"d": "Three", "b": {}} # exists
+          >>> document.set(document_data, merge=True)
+          >>> document.get().to_dict() == {"a": 1, "d": "Three", "b": {"c": "Two"}}
+          True
+
+          In this case, existing documents with top-level keys which are
+          not present in ``document_data`` (``"d"``) will preserve the values
+          of those keys.
+
+
+        - To merge only specific fields of ``document_data`` with existing
+          documents (creating if the document does not exist), pass ``merge``
+          as a list of field paths:
+
+
+          >>> document_data = {"a": 1, "b": {"c": "Two"}}
+          >>> document.get().to_dict() == {"b": {"c": "One", "d": "Four" }} # exists
+          True
+          >>> document.set(document_data, merge=["b.c"])
+          >>> document.get().to_dict() == {"b": {"c": "Two", "d": "Four" }}
+          True
+
+          For more information on field paths, see
+          :meth:`~google.cloud.firestore_v1.base_client.BaseClient.field_path`.
 
         Args:
             document_data (dict): Property names and values to use for
                 replacing a document.
-            merge (Optional[bool] or Optional[List<apispec>]):
+            merge (Optional[bool] or Optional[List<fieldpath>]):
                 If True, apply merging instead of overwriting the state
                 of the document.
             retry (google.api_core.retry.Retry): Designation of what errors, if any,
@@ -142,9 +181,9 @@ class DocumentReference(BaseDocumentReference):
         override these preconditions.
 
         Each key in ``field_updates`` can either be a field name or a
-        **field path** (For more information on **field paths**, see
-        :meth:`~google.cloud.firestore_v1.client.Client.field_path`.) To
-        illustrate this, consider a document with
+        **field path** (For more information on field paths, see
+        :meth:`~google.cloud.firestore_v1.base_client.BaseClient.field_path`.)
+        To illustrate this, consider a document with
 
         .. code-block:: python
 
@@ -290,7 +329,7 @@ class DocumentReference(BaseDocumentReference):
         option: _helpers.WriteOption = None,
         retry: retries.Retry = gapic_v1.method.DEFAULT,
         timeout: float = None,
-    ) -> timestamp_pb2.Timestamp:
+    ) -> Timestamp:
         """Delete the current document in the Firestore database.
 
         Args:
@@ -354,31 +393,35 @@ class DocumentReference(BaseDocumentReference):
                 :attr:`create_time` attributes will all be ``None`` and
                 its :attr:`exists` attribute will be ``False``.
         """
-        request, kwargs = self._prep_get(field_paths, transaction, retry, timeout)
+        from google.cloud.firestore_v1.base_client import _parse_batch_get
 
-        firestore_api = self._client._firestore_api
-        try:
-            document_pb = firestore_api.get_document(
-                request=request, metadata=self._client._rpc_metadata, **kwargs,
+        request, kwargs = self._prep_batch_get(field_paths, transaction, retry, timeout)
+
+        response_iter = self._client._firestore_api.batch_get_documents(
+            request=request, metadata=self._client._rpc_metadata, **kwargs,
+        )
+
+        get_doc_response = next(response_iter, None)
+
+        if get_doc_response is not None:
+            return _parse_batch_get(
+                get_doc_response=get_doc_response,
+                reference_map={self._document_path: self},
+                client=self._client,
             )
-        except exceptions.NotFound:
-            data = None
-            exists = False
-            create_time = None
-            update_time = None
-        else:
-            data = _helpers.decode_dict(document_pb.fields, self._client)
-            exists = True
-            create_time = document_pb.create_time
-            update_time = document_pb.update_time
+
+        logger.warning(
+            "`batch_get_documents` unexpectedly returned empty "
+            "stream. Expected one object.",
+        )
 
         return DocumentSnapshot(
-            reference=self,
-            data=data,
-            exists=exists,
-            read_time=None,  # No server read_time available
-            create_time=create_time,
-            update_time=update_time,
+            self,
+            None,
+            exists=False,
+            read_time=_datetime_to_pb_timestamp(datetime.datetime.now()),
+            create_time=None,
+            update_time=None,
         )
 
     def collections(
@@ -420,7 +463,7 @@ class DocumentReference(BaseDocumentReference):
         provided callback is run on the snapshot.
 
         Args:
-            callback(Callable[[:class:`~google.cloud.firestore.document.DocumentSnapshot`], NoneType]):
+            callback(Callable[[:class:`~google.cloud.firestore_v1.base_document.DocumentSnapshot`], NoneType]):
                 a callback to run when a change occurs
 
         Example:
