@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,22 +13,22 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import abc
-import typing
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Union
+import packaging.version
 import pkg_resources
 
-from google import auth  # type: ignore
-from google.api_core import exceptions  # type: ignore
+import google.auth  # type: ignore
+import google.api_core  # type: ignore
+from google.api_core import exceptions as core_exceptions  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import retry as retries  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 
 from google.cloud.firestore_v1.types import document
 from google.cloud.firestore_v1.types import document as gf_document
 from google.cloud.firestore_v1.types import firestore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 
 try:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
@@ -38,6 +37,17 @@ try:
 except pkg_resources.DistributionNotFound:
     DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo()
 
+try:
+    # google.auth.__version__ was added in 1.26.0
+    _GOOGLE_AUTH_VERSION = google.auth.__version__
+except AttributeError:
+    try:  # try pkg_resources if it is available
+        _GOOGLE_AUTH_VERSION = pkg_resources.get_distribution("google-auth").version
+    except pkg_resources.DistributionNotFound:  # pragma: NO COVER
+        _GOOGLE_AUTH_VERSION = None
+
+_API_CORE_VERSION = google.api_core.__version__
+
 
 class FirestoreTransport(abc.ABC):
     """Abstract transport class for Firestore."""
@@ -47,21 +57,24 @@ class FirestoreTransport(abc.ABC):
         "https://www.googleapis.com/auth/datastore",
     )
 
+    DEFAULT_HOST: str = "firestore.googleapis.com"
+
     def __init__(
         self,
         *,
-        host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
-        credentials_file: typing.Optional[str] = None,
-        scopes: typing.Optional[typing.Sequence[str]] = AUTH_SCOPES,
-        quota_project_id: typing.Optional[str] = None,
+        host: str = DEFAULT_HOST,
+        credentials: ga_credentials.Credentials = None,
+        credentials_file: Optional[str] = None,
+        scopes: Optional[Sequence[str]] = None,
+        quota_project_id: Optional[str] = None,
         client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
         **kwargs,
     ) -> None:
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -70,7 +83,7 @@ class FirestoreTransport(abc.ABC):
             credentials_file (Optional[str]): A file with credentials that can
                 be loaded with :func:`google.auth.load_credentials_from_file`.
                 This argument is mutually exclusive with credentials.
-            scope (Optional[Sequence[str]]): A list of scopes.
+            scopes (Optional[Sequence[str]]): A list of scopes.
             quota_project_id (Optional[str]): An optional project to use for billing
                 and quota.
             client_info (google.api_core.gapic_v1.client_info.ClientInfo):
@@ -84,29 +97,76 @@ class FirestoreTransport(abc.ABC):
             host += ":443"
         self._host = host
 
+        scopes_kwargs = self._get_scopes_kwargs(self._host, scopes)
+
         # Save the scopes.
         self._scopes = scopes or self.AUTH_SCOPES
 
         # If no credentials are provided, then determine the appropriate
         # defaults.
         if credentials and credentials_file:
-            raise exceptions.DuplicateCredentialArgs(
+            raise core_exceptions.DuplicateCredentialArgs(
                 "'credentials_file' and 'credentials' are mutually exclusive"
             )
 
         if credentials_file is not None:
-            credentials, _ = auth.load_credentials_from_file(
-                credentials_file, scopes=self._scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.load_credentials_from_file(
+                credentials_file, **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         elif credentials is None:
-            credentials, _ = auth.default(
-                scopes=self._scopes, quota_project_id=quota_project_id
+            credentials, _ = google.auth.default(
+                **scopes_kwargs, quota_project_id=quota_project_id
             )
 
         # Save the credentials.
         self._credentials = credentials
 
+    # TODO(busunkim): These two class methods are in the base transport
+    # to avoid duplicating code across the transport classes. These functions
+    # should be deleted once the minimum required versions of google-api-core
+    # and google-auth are increased.
+
+    # TODO: Remove this function once google-auth >= 1.25.0 is required
+    @classmethod
+    def _get_scopes_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Optional[Sequence[str]]]:
+        """Returns scopes kwargs to pass to google-auth methods depending on the google-auth version"""
+
+        scopes_kwargs = {}
+
+        if _GOOGLE_AUTH_VERSION and (
+            packaging.version.parse(_GOOGLE_AUTH_VERSION)
+            >= packaging.version.parse("1.25.0")
+        ):
+            scopes_kwargs = {"scopes": scopes, "default_scopes": cls.AUTH_SCOPES}
+        else:
+            scopes_kwargs = {"scopes": scopes or cls.AUTH_SCOPES}
+
+        return scopes_kwargs
+
+    # TODO: Remove this function once google-api-core >= 1.26.0 is required
+    @classmethod
+    def _get_self_signed_jwt_kwargs(
+        cls, host: str, scopes: Optional[Sequence[str]]
+    ) -> Dict[str, Union[Optional[Sequence[str]], str]]:
+        """Returns kwargs to pass to grpc_helpers.create_channel depending on the google-api-core version"""
+
+        self_signed_jwt_kwargs: Dict[str, Union[Optional[Sequence[str]], str]] = {}
+
+        if _API_CORE_VERSION and (
+            packaging.version.parse(_API_CORE_VERSION)
+            >= packaging.version.parse("1.26.0")
+        ):
+            self_signed_jwt_kwargs["default_scopes"] = cls.AUTH_SCOPES
+            self_signed_jwt_kwargs["scopes"] = scopes
+            self_signed_jwt_kwargs["default_host"] = cls.DEFAULT_HOST
+        else:
+            self_signed_jwt_kwargs["scopes"] = scopes or cls.AUTH_SCOPES
+
+        return self_signed_jwt_kwargs
+
     def _prep_wrapped_messages(self, client_info):
         # Precompute the wrapped methods.
         self._wrapped_methods = {
@@ -117,9 +177,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -133,9 +194,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -148,7 +210,10 @@ class FirestoreTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
+                    ),
                     deadline=60.0,
                 ),
                 default_timeout=60.0,
@@ -161,9 +226,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -177,9 +243,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=300.0,
                 ),
@@ -193,9 +260,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -208,7 +276,10 @@ class FirestoreTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
+                    ),
                     deadline=60.0,
                 ),
                 default_timeout=60.0,
@@ -221,9 +292,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -237,9 +309,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=300.0,
                 ),
@@ -253,9 +326,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=300.0,
                 ),
@@ -272,9 +346,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=86400.0,
                 ),
@@ -288,9 +363,10 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.DeadlineExceeded,
-                        exceptions.InternalServerError,
-                        exceptions.ServiceUnavailable,
+                        core_exceptions.DeadlineExceeded,
+                        core_exceptions.InternalServerError,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -304,7 +380,9 @@ class FirestoreTransport(abc.ABC):
                     maximum=60.0,
                     multiplier=1.3,
                     predicate=retries.if_exception_type(
-                        exceptions.Aborted, exceptions.ServiceUnavailable,
+                        core_exceptions.Aborted,
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
                     ),
                     deadline=60.0,
                 ),
@@ -317,7 +395,10 @@ class FirestoreTransport(abc.ABC):
                     initial=0.1,
                     maximum=60.0,
                     multiplier=1.3,
-                    predicate=retries.if_exception_type(exceptions.ServiceUnavailable,),
+                    predicate=retries.if_exception_type(
+                        core_exceptions.ResourceExhausted,
+                        core_exceptions.ServiceUnavailable,
+                    ),
                     deadline=60.0,
                 ),
                 default_timeout=60.0,
@@ -328,20 +409,19 @@ class FirestoreTransport(abc.ABC):
     @property
     def get_document(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.GetDocumentRequest],
-        typing.Union[document.Document, typing.Awaitable[document.Document]],
+        Union[document.Document, Awaitable[document.Document]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_documents(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.ListDocumentsRequest],
-        typing.Union[
-            firestore.ListDocumentsResponse,
-            typing.Awaitable[firestore.ListDocumentsResponse],
+        Union[
+            firestore.ListDocumentsResponse, Awaitable[firestore.ListDocumentsResponse]
         ],
     ]:
         raise NotImplementedError()
@@ -349,29 +429,29 @@ class FirestoreTransport(abc.ABC):
     @property
     def update_document(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.UpdateDocumentRequest],
-        typing.Union[gf_document.Document, typing.Awaitable[gf_document.Document]],
+        Union[gf_document.Document, Awaitable[gf_document.Document]],
     ]:
         raise NotImplementedError()
 
     @property
     def delete_document(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.DeleteDocumentRequest],
-        typing.Union[empty.Empty, typing.Awaitable[empty.Empty]],
+        Union[empty_pb2.Empty, Awaitable[empty_pb2.Empty]],
     ]:
         raise NotImplementedError()
 
     @property
     def batch_get_documents(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.BatchGetDocumentsRequest],
-        typing.Union[
+        Union[
             firestore.BatchGetDocumentsResponse,
-            typing.Awaitable[firestore.BatchGetDocumentsResponse],
+            Awaitable[firestore.BatchGetDocumentsResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -379,11 +459,11 @@ class FirestoreTransport(abc.ABC):
     @property
     def begin_transaction(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.BeginTransactionRequest],
-        typing.Union[
+        Union[
             firestore.BeginTransactionResponse,
-            typing.Awaitable[firestore.BeginTransactionResponse],
+            Awaitable[firestore.BeginTransactionResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -391,42 +471,37 @@ class FirestoreTransport(abc.ABC):
     @property
     def commit(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.CommitRequest],
-        typing.Union[
-            firestore.CommitResponse, typing.Awaitable[firestore.CommitResponse]
-        ],
+        Union[firestore.CommitResponse, Awaitable[firestore.CommitResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def rollback(
         self,
-    ) -> typing.Callable[
-        [firestore.RollbackRequest],
-        typing.Union[empty.Empty, typing.Awaitable[empty.Empty]],
+    ) -> Callable[
+        [firestore.RollbackRequest], Union[empty_pb2.Empty, Awaitable[empty_pb2.Empty]]
     ]:
         raise NotImplementedError()
 
     @property
     def run_query(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.RunQueryRequest],
-        typing.Union[
-            firestore.RunQueryResponse, typing.Awaitable[firestore.RunQueryResponse]
-        ],
+        Union[firestore.RunQueryResponse, Awaitable[firestore.RunQueryResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def partition_query(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.PartitionQueryRequest],
-        typing.Union[
+        Union[
             firestore.PartitionQueryResponse,
-            typing.Awaitable[firestore.PartitionQueryResponse],
+            Awaitable[firestore.PartitionQueryResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -434,33 +509,29 @@ class FirestoreTransport(abc.ABC):
     @property
     def write(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.WriteRequest],
-        typing.Union[
-            firestore.WriteResponse, typing.Awaitable[firestore.WriteResponse]
-        ],
+        Union[firestore.WriteResponse, Awaitable[firestore.WriteResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def listen(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.ListenRequest],
-        typing.Union[
-            firestore.ListenResponse, typing.Awaitable[firestore.ListenResponse]
-        ],
+        Union[firestore.ListenResponse, Awaitable[firestore.ListenResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def list_collection_ids(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.ListCollectionIdsRequest],
-        typing.Union[
+        Union[
             firestore.ListCollectionIdsResponse,
-            typing.Awaitable[firestore.ListCollectionIdsResponse],
+            Awaitable[firestore.ListCollectionIdsResponse],
         ],
     ]:
         raise NotImplementedError()
@@ -468,20 +539,18 @@ class FirestoreTransport(abc.ABC):
     @property
     def batch_write(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.BatchWriteRequest],
-        typing.Union[
-            firestore.BatchWriteResponse, typing.Awaitable[firestore.BatchWriteResponse]
-        ],
+        Union[firestore.BatchWriteResponse, Awaitable[firestore.BatchWriteResponse]],
     ]:
         raise NotImplementedError()
 
     @property
     def create_document(
         self,
-    ) -> typing.Callable[
+    ) -> Callable[
         [firestore.CreateDocumentRequest],
-        typing.Union[document.Document, typing.Awaitable[document.Document]],
+        Union[document.Document, Awaitable[document.Document]],
     ]:
         raise NotImplementedError()
 
