@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,14 +13,13 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Callable, Dict, Optional, Sequence, Tuple
+from typing import Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import grpc_helpers  # type: ignore
 from google.api_core import gapic_v1  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+import google.auth  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
 
 import grpc  # type: ignore
@@ -29,8 +27,7 @@ import grpc  # type: ignore
 from google.cloud.firestore_v1.types import document
 from google.cloud.firestore_v1.types import document as gf_document
 from google.cloud.firestore_v1.types import firestore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 from .base import FirestoreTransport, DEFAULT_CLIENT_INFO
 
 
@@ -60,7 +57,7 @@ class FirestoreGrpcTransport(FirestoreTransport):
         self,
         *,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Sequence[str] = None,
         channel: grpc.Channel = None,
@@ -74,7 +71,8 @@ class FirestoreGrpcTransport(FirestoreTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -184,7 +182,7 @@ class FirestoreGrpcTransport(FirestoreTransport):
     def create_channel(
         cls,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: str = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -215,13 +213,15 @@ class FirestoreGrpcTransport(FirestoreTransport):
             google.api_core.exceptions.DuplicateCredentialArgs: If both ``credentials``
               and ``credentials_file`` are passed.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -312,7 +312,7 @@ class FirestoreGrpcTransport(FirestoreTransport):
     @property
     def delete_document(
         self,
-    ) -> Callable[[firestore.DeleteDocumentRequest], empty.Empty]:
+    ) -> Callable[[firestore.DeleteDocumentRequest], empty_pb2.Empty]:
         r"""Return a callable for the delete document method over gRPC.
 
         Deletes a document.
@@ -331,7 +331,7 @@ class FirestoreGrpcTransport(FirestoreTransport):
             self._stubs["delete_document"] = self.grpc_channel.unary_unary(
                 "/google.firestore.v1.Firestore/DeleteDocument",
                 request_serializer=firestore.DeleteDocumentRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["delete_document"]
 
@@ -419,7 +419,7 @@ class FirestoreGrpcTransport(FirestoreTransport):
         return self._stubs["commit"]
 
     @property
-    def rollback(self) -> Callable[[firestore.RollbackRequest], empty.Empty]:
+    def rollback(self) -> Callable[[firestore.RollbackRequest], empty_pb2.Empty]:
         r"""Return a callable for the rollback method over gRPC.
 
         Rolls back a transaction.
@@ -438,7 +438,7 @@ class FirestoreGrpcTransport(FirestoreTransport):
             self._stubs["rollback"] = self.grpc_channel.unary_unary(
                 "/google.firestore.v1.Firestore/Rollback",
                 request_serializer=firestore.RollbackRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["rollback"]
 
