@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,15 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import warnings
-from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple
+from typing import Awaitable, Callable, Dict, Optional, Sequence, Tuple, Union
 
 from google.api_core import gapic_v1  # type: ignore
 from google.api_core import grpc_helpers_async  # type: ignore
-from google import auth  # type: ignore
-from google.auth import credentials  # type: ignore
+from google.auth import credentials as ga_credentials  # type: ignore
 from google.auth.transport.grpc import SslCredentials  # type: ignore
+import packaging.version
 
 import grpc  # type: ignore
 from grpc.experimental import aio  # type: ignore
@@ -30,8 +28,7 @@ from grpc.experimental import aio  # type: ignore
 from google.cloud.firestore_v1.types import document
 from google.cloud.firestore_v1.types import document as gf_document
 from google.cloud.firestore_v1.types import firestore
-from google.protobuf import empty_pb2 as empty  # type: ignore
-
+from google.protobuf import empty_pb2  # type: ignore
 from .base import FirestoreTransport, DEFAULT_CLIENT_INFO
 from .grpc import FirestoreGrpcTransport
 
@@ -63,7 +60,7 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
     def create_channel(
         cls,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         quota_project_id: Optional[str] = None,
@@ -90,13 +87,15 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
         Returns:
             aio.Channel: A gRPC AsyncIO channel object.
         """
-        scopes = scopes or cls.AUTH_SCOPES
+
+        self_signed_jwt_kwargs = cls._get_self_signed_jwt_kwargs(host, scopes)
+
         return grpc_helpers_async.create_channel(
             host,
             credentials=credentials,
             credentials_file=credentials_file,
-            scopes=scopes,
             quota_project_id=quota_project_id,
+            **self_signed_jwt_kwargs,
             **kwargs,
         )
 
@@ -104,7 +103,7 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
         self,
         *,
         host: str = "firestore.googleapis.com",
-        credentials: credentials.Credentials = None,
+        credentials: ga_credentials.Credentials = None,
         credentials_file: Optional[str] = None,
         scopes: Optional[Sequence[str]] = None,
         channel: aio.Channel = None,
@@ -118,7 +117,8 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
         """Instantiate the transport.
 
         Args:
-            host (Optional[str]): The hostname to connect to.
+            host (Optional[str]):
+                 The hostname to connect to.
             credentials (Optional[google.auth.credentials.Credentials]): The
                 authorization credentials to attach to requests. These
                 credentials identify the application to the service; if none
@@ -176,7 +176,6 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
             # If a channel was explicitly provided, set it.
             self._grpc_channel = channel
             self._ssl_channel_credentials = None
-
         else:
             if api_mtls_endpoint:
                 host = api_mtls_endpoint
@@ -318,7 +317,7 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
     @property
     def delete_document(
         self,
-    ) -> Callable[[firestore.DeleteDocumentRequest], Awaitable[empty.Empty]]:
+    ) -> Callable[[firestore.DeleteDocumentRequest], Awaitable[empty_pb2.Empty]]:
         r"""Return a callable for the delete document method over gRPC.
 
         Deletes a document.
@@ -337,7 +336,7 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
             self._stubs["delete_document"] = self.grpc_channel.unary_unary(
                 "/google.firestore.v1.Firestore/DeleteDocument",
                 request_serializer=firestore.DeleteDocumentRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["delete_document"]
 
@@ -429,7 +428,9 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
         return self._stubs["commit"]
 
     @property
-    def rollback(self) -> Callable[[firestore.RollbackRequest], Awaitable[empty.Empty]]:
+    def rollback(
+        self,
+    ) -> Callable[[firestore.RollbackRequest], Awaitable[empty_pb2.Empty]]:
         r"""Return a callable for the rollback method over gRPC.
 
         Rolls back a transaction.
@@ -448,7 +449,7 @@ class FirestoreGrpcAsyncIOTransport(FirestoreTransport):
             self._stubs["rollback"] = self.grpc_channel.unary_unary(
                 "/google.firestore.v1.Firestore/Rollback",
                 request_serializer=firestore.RollbackRequest.serialize,
-                response_deserializer=empty.Empty.FromString,
+                response_deserializer=empty_pb2.Empty.FromString,
             )
         return self._stubs["rollback"]
 
