@@ -1,5 +1,4 @@
 # -*- coding: utf-8 -*-
-
 # Copyright 2020 Google LLC
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
@@ -14,16 +13,14 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 #
-
 import proto  # type: ignore
 
-
 from google.cloud.firestore_v1.types import common
 from google.cloud.firestore_v1.types import document as gf_document
 from google.cloud.firestore_v1.types import query as gf_query
 from google.cloud.firestore_v1.types import write
-from google.protobuf import timestamp_pb2 as timestamp  # type: ignore
-from google.rpc import status_pb2 as gr_status  # type: ignore
+from google.protobuf import timestamp_pb2  # type: ignore
+from google.rpc import status_pb2  # type: ignore
 
 
 __protobuf__ = proto.module(
@@ -83,17 +80,14 @@ class GetDocumentRequest(proto.Message):
             seconds.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
+    name = proto.Field(proto.STRING, number=1,)
     mask = proto.Field(proto.MESSAGE, number=2, message=common.DocumentMask,)
-
-    transaction = proto.Field(proto.BYTES, number=3, oneof="consistency_selector")
-
+    transaction = proto.Field(proto.BYTES, number=3, oneof="consistency_selector",)
     read_time = proto.Field(
         proto.MESSAGE,
         number=5,
         oneof="consistency_selector",
-        message=timestamp.Timestamp,
+        message=timestamp_pb2.Timestamp,
     )
 
 
@@ -146,28 +140,20 @@ class ListDocumentsRequest(proto.Message):
             ``order_by``.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    collection_id = proto.Field(proto.STRING, number=2)
-
-    page_size = proto.Field(proto.INT32, number=3)
-
-    page_token = proto.Field(proto.STRING, number=4)
-
-    order_by = proto.Field(proto.STRING, number=6)
-
+    parent = proto.Field(proto.STRING, number=1,)
+    collection_id = proto.Field(proto.STRING, number=2,)
+    page_size = proto.Field(proto.INT32, number=3,)
+    page_token = proto.Field(proto.STRING, number=4,)
+    order_by = proto.Field(proto.STRING, number=6,)
     mask = proto.Field(proto.MESSAGE, number=7, message=common.DocumentMask,)
-
-    transaction = proto.Field(proto.BYTES, number=8, oneof="consistency_selector")
-
+    transaction = proto.Field(proto.BYTES, number=8, oneof="consistency_selector",)
     read_time = proto.Field(
         proto.MESSAGE,
         number=10,
         oneof="consistency_selector",
-        message=timestamp.Timestamp,
+        message=timestamp_pb2.Timestamp,
     )
-
-    show_missing = proto.Field(proto.BOOL, number=12)
+    show_missing = proto.Field(proto.BOOL, number=12,)
 
 
 class ListDocumentsResponse(proto.Message):
@@ -188,8 +174,7 @@ class ListDocumentsResponse(proto.Message):
     documents = proto.RepeatedField(
         proto.MESSAGE, number=1, message=gf_document.Document,
     )
-
-    next_page_token = proto.Field(proto.STRING, number=2)
+    next_page_token = proto.Field(proto.STRING, number=2,)
 
 
 class CreateDocumentRequest(proto.Message):
@@ -220,14 +205,10 @@ class CreateDocumentRequest(proto.Message):
             the response.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    collection_id = proto.Field(proto.STRING, number=2)
-
-    document_id = proto.Field(proto.STRING, number=3)
-
+    parent = proto.Field(proto.STRING, number=1,)
+    collection_id = proto.Field(proto.STRING, number=2,)
+    document_id = proto.Field(proto.STRING, number=3,)
     document = proto.Field(proto.MESSAGE, number=4, message=gf_document.Document,)
-
     mask = proto.Field(proto.MESSAGE, number=5, message=common.DocumentMask,)
 
 
@@ -263,11 +244,8 @@ class UpdateDocumentRequest(proto.Message):
     """
 
     document = proto.Field(proto.MESSAGE, number=1, message=gf_document.Document,)
-
     update_mask = proto.Field(proto.MESSAGE, number=2, message=common.DocumentMask,)
-
     mask = proto.Field(proto.MESSAGE, number=3, message=common.DocumentMask,)
-
     current_document = proto.Field(
         proto.MESSAGE, number=4, message=common.Precondition,
     )
@@ -288,8 +266,7 @@ class DeleteDocumentRequest(proto.Message):
             by the target document.
     """
 
-    name = proto.Field(proto.STRING, number=1)
-
+    name = proto.Field(proto.STRING, number=1,)
     current_document = proto.Field(
         proto.MESSAGE, number=2, message=common.Precondition,
     )
@@ -327,26 +304,21 @@ class BatchGetDocumentsRequest(proto.Message):
             time. This may not be older than 270 seconds.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
-    documents = proto.RepeatedField(proto.STRING, number=2)
-
+    database = proto.Field(proto.STRING, number=1,)
+    documents = proto.RepeatedField(proto.STRING, number=2,)
     mask = proto.Field(proto.MESSAGE, number=3, message=common.DocumentMask,)
-
-    transaction = proto.Field(proto.BYTES, number=4, oneof="consistency_selector")
-
+    transaction = proto.Field(proto.BYTES, number=4, oneof="consistency_selector",)
     new_transaction = proto.Field(
         proto.MESSAGE,
         number=5,
         oneof="consistency_selector",
         message=common.TransactionOptions,
     )
-
     read_time = proto.Field(
         proto.MESSAGE,
         number=7,
         oneof="consistency_selector",
-        message=timestamp.Timestamp,
+        message=timestamp_pb2.Timestamp,
     )
 
 
@@ -376,12 +348,9 @@ class BatchGetDocumentsResponse(proto.Message):
     found = proto.Field(
         proto.MESSAGE, number=1, oneof="result", message=gf_document.Document,
     )
-
-    missing = proto.Field(proto.STRING, number=2, oneof="result")
-
-    transaction = proto.Field(proto.BYTES, number=3)
-
-    read_time = proto.Field(proto.MESSAGE, number=4, message=timestamp.Timestamp,)
+    missing = proto.Field(proto.STRING, number=2, oneof="result",)
+    transaction = proto.Field(proto.BYTES, number=3,)
+    read_time = proto.Field(proto.MESSAGE, number=4, message=timestamp_pb2.Timestamp,)
 
 
 class BeginTransactionRequest(proto.Message):
@@ -397,8 +366,7 @@ class BeginTransactionRequest(proto.Message):
             Defaults to a read-write transaction.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
+    database = proto.Field(proto.STRING, number=1,)
     options = proto.Field(proto.MESSAGE, number=2, message=common.TransactionOptions,)
 
 
@@ -411,7 +379,7 @@ class BeginTransactionResponse(proto.Message):
             The transaction that was started.
     """
 
-    transaction = proto.Field(proto.BYTES, number=1)
+    transaction = proto.Field(proto.BYTES, number=1,)
 
 
 class CommitRequest(proto.Message):
@@ -430,11 +398,9 @@ class CommitRequest(proto.Message):
             transaction, and commits it.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
+    database = proto.Field(proto.STRING, number=1,)
     writes = proto.RepeatedField(proto.MESSAGE, number=2, message=write.Write,)
-
-    transaction = proto.Field(proto.BYTES, number=3)
+    transaction = proto.Field(proto.BYTES, number=3,)
 
 
 class CommitResponse(proto.Message):
@@ -455,8 +421,7 @@ class CommitResponse(proto.Message):
     write_results = proto.RepeatedField(
         proto.MESSAGE, number=1, message=write.WriteResult,
     )
-
-    commit_time = proto.Field(proto.MESSAGE, number=2, message=timestamp.Timestamp,)
+    commit_time = proto.Field(proto.MESSAGE, number=2, message=timestamp_pb2.Timestamp,)
 
 
 class RollbackRequest(proto.Message):
@@ -471,9 +436,8 @@ class RollbackRequest(proto.Message):
             Required. The transaction to roll back.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
-    transaction = proto.Field(proto.BYTES, number=2)
+    database = proto.Field(proto.STRING, number=1,)
+    transaction = proto.Field(proto.BYTES, number=2,)
 
 
 class RunQueryRequest(proto.Message):
@@ -503,26 +467,22 @@ class RunQueryRequest(proto.Message):
             time. This may not be older than 270 seconds.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
+    parent = proto.Field(proto.STRING, number=1,)
     structured_query = proto.Field(
         proto.MESSAGE, number=2, oneof="query_type", message=gf_query.StructuredQuery,
     )
-
-    transaction = proto.Field(proto.BYTES, number=5, oneof="consistency_selector")
-
+    transaction = proto.Field(proto.BYTES, number=5, oneof="consistency_selector",)
     new_transaction = proto.Field(
         proto.MESSAGE,
         number=6,
         oneof="consistency_selector",
         message=common.TransactionOptions,
     )
-
     read_time = proto.Field(
         proto.MESSAGE,
         number=7,
         oneof="consistency_selector",
-        message=timestamp.Timestamp,
+        message=timestamp_pb2.Timestamp,
     )
 
 
@@ -555,13 +515,10 @@ class RunQueryResponse(proto.Message):
             the current response.
     """
 
-    transaction = proto.Field(proto.BYTES, number=2)
-
+    transaction = proto.Field(proto.BYTES, number=2,)
     document = proto.Field(proto.MESSAGE, number=1, message=gf_document.Document,)
-
-    read_time = proto.Field(proto.MESSAGE, number=3, message=timestamp.Timestamp,)
-
-    skipped_results = proto.Field(proto.INT32, number=4)
+    read_time = proto.Field(proto.MESSAGE, number=3, message=timestamp_pb2.Timestamp,)
+    skipped_results = proto.Field(proto.INT32, number=4,)
 
 
 class PartitionQueryRequest(proto.Message):
@@ -621,17 +578,13 @@ class PartitionQueryRequest(proto.Message):
             ``partition_count``.
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
+    parent = proto.Field(proto.STRING, number=1,)
     structured_query = proto.Field(
         proto.MESSAGE, number=2, oneof="query_type", message=gf_query.StructuredQuery,
     )
-
-    partition_count = proto.Field(proto.INT64, number=3)
-
-    page_token = proto.Field(proto.STRING, number=4)
-
-    page_size = proto.Field(proto.INT32, number=5)
+    partition_count = proto.Field(proto.INT64, number=3,)
+    page_token = proto.Field(proto.STRING, number=4,)
+    page_size = proto.Field(proto.INT32, number=5,)
 
 
 class PartitionQueryResponse(proto.Message):
@@ -669,8 +622,7 @@ class PartitionQueryResponse(proto.Message):
         return self
 
     partitions = proto.RepeatedField(proto.MESSAGE, number=1, message=gf_query.Cursor,)
-
-    next_page_token = proto.Field(proto.STRING, number=2)
+    next_page_token = proto.Field(proto.STRING, number=2,)
 
 
 class WriteRequest(proto.Message):
@@ -723,15 +675,11 @@ class WriteRequest(proto.Message):
             Labels associated with this write request.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
-    stream_id = proto.Field(proto.STRING, number=2)
-
+    database = proto.Field(proto.STRING, number=1,)
+    stream_id = proto.Field(proto.STRING, number=2,)
     writes = proto.RepeatedField(proto.MESSAGE, number=3, message=write.Write,)
-
-    stream_token = proto.Field(proto.BYTES, number=4)
-
-    labels = proto.MapField(proto.STRING, proto.STRING, number=5)
+    stream_token = proto.Field(proto.BYTES, number=4,)
+    labels = proto.MapField(proto.STRING, proto.STRING, number=5,)
 
 
 class WriteResponse(proto.Message):
@@ -758,15 +706,12 @@ class WriteResponse(proto.Message):
             effects of the write.
     """
 
-    stream_id = proto.Field(proto.STRING, number=1)
-
-    stream_token = proto.Field(proto.BYTES, number=2)
-
+    stream_id = proto.Field(proto.STRING, number=1,)
+    stream_token = proto.Field(proto.BYTES, number=2,)
     write_results = proto.RepeatedField(
         proto.MESSAGE, number=3, message=write.WriteResult,
     )
-
-    commit_time = proto.Field(proto.MESSAGE, number=4, message=timestamp.Timestamp,)
+    commit_time = proto.Field(proto.MESSAGE, number=4, message=timestamp_pb2.Timestamp,)
 
 
 class ListenRequest(proto.Message):
@@ -786,15 +731,12 @@ class ListenRequest(proto.Message):
             Labels associated with this target change.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
+    database = proto.Field(proto.STRING, number=1,)
     add_target = proto.Field(
         proto.MESSAGE, number=2, oneof="target_change", message="Target",
     )
-
-    remove_target = proto.Field(proto.INT32, number=3, oneof="target_change")
-
-    labels = proto.MapField(proto.STRING, proto.STRING, number=4)
+    remove_target = proto.Field(proto.INT32, number=3, oneof="target_change",)
+    labels = proto.MapField(proto.STRING, proto.STRING, number=4,)
 
 
 class ListenResponse(proto.Message):
@@ -824,19 +766,15 @@ class ListenResponse(proto.Message):
     target_change = proto.Field(
         proto.MESSAGE, number=2, oneof="response_type", message="TargetChange",
     )
-
     document_change = proto.Field(
         proto.MESSAGE, number=3, oneof="response_type", message=write.DocumentChange,
     )
-
     document_delete = proto.Field(
         proto.MESSAGE, number=4, oneof="response_type", message=write.DocumentDelete,
     )
-
     document_remove = proto.Field(
         proto.MESSAGE, number=6, oneof="response_type", message=write.DocumentRemove,
     )
-
     filter = proto.Field(
         proto.MESSAGE, number=5, oneof="response_type", message=write.ExistenceFilter,
     )
@@ -844,7 +782,6 @@ class ListenResponse(proto.Message):
 
 class Target(proto.Message):
     r"""A specification of a set of documents to listen to.
-
     Attributes:
         query (google.cloud.firestore_v1.types.Target.QueryTarget):
             A target specified by a query.
@@ -874,7 +811,6 @@ class Target(proto.Message):
 
     class DocumentsTarget(proto.Message):
         r"""A target specified by a set of documents names.
-
         Attributes:
             documents (Sequence[str]):
                 The names of the documents to retrieve. In the format:
@@ -884,11 +820,10 @@ class Target(proto.Message):
                 elided.
         """
 
-        documents = proto.RepeatedField(proto.STRING, number=2)
+        documents = proto.RepeatedField(proto.STRING, number=2,)
 
     class QueryTarget(proto.Message):
         r"""A target specified by a query.
-
         Attributes:
             parent (str):
                 The parent resource name. In the format:
@@ -902,8 +837,7 @@ class Target(proto.Message):
                 A structured query.
         """
 
-        parent = proto.Field(proto.STRING, number=1)
-
+        parent = proto.Field(proto.STRING, number=1,)
         structured_query = proto.Field(
             proto.MESSAGE,
             number=2,
@@ -914,25 +848,19 @@ class Target(proto.Message):
     query = proto.Field(
         proto.MESSAGE, number=2, oneof="target_type", message=QueryTarget,
     )
-
     documents = proto.Field(
         proto.MESSAGE, number=3, oneof="target_type", message=DocumentsTarget,
     )
-
-    resume_token = proto.Field(proto.BYTES, number=4, oneof="resume_type")
-
+    resume_token = proto.Field(proto.BYTES, number=4, oneof="resume_type",)
     read_time = proto.Field(
-        proto.MESSAGE, number=11, oneof="resume_type", message=timestamp.Timestamp,
+        proto.MESSAGE, number=11, oneof="resume_type", message=timestamp_pb2.Timestamp,
     )
-
-    target_id = proto.Field(proto.INT32, number=5)
-
-    once = proto.Field(proto.BOOL, number=6)
+    target_id = proto.Field(proto.INT32, number=5,)
+    once = proto.Field(proto.BOOL, number=6,)
 
 
 class TargetChange(proto.Message):
     r"""Targets being watched have changed.
-
     Attributes:
         target_change_type (google.cloud.firestore_v1.types.TargetChange.TargetChangeType):
             The type of change that occurred.
@@ -973,14 +901,10 @@ class TargetChange(proto.Message):
         RESET = 4
 
     target_change_type = proto.Field(proto.ENUM, number=1, enum=TargetChangeType,)
-
-    target_ids = proto.RepeatedField(proto.INT32, number=2)
-
-    cause = proto.Field(proto.MESSAGE, number=3, message=gr_status.Status,)
-
-    resume_token = proto.Field(proto.BYTES, number=4)
-
-    read_time = proto.Field(proto.MESSAGE, number=6, message=timestamp.Timestamp,)
+    target_ids = proto.RepeatedField(proto.INT32, number=2,)
+    cause = proto.Field(proto.MESSAGE, number=3, message=status_pb2.Status,)
+    resume_token = proto.Field(proto.BYTES, number=4,)
+    read_time = proto.Field(proto.MESSAGE, number=6, message=timestamp_pb2.Timestamp,)
 
 
 class ListCollectionIdsRequest(proto.Message):
@@ -1000,11 +924,9 @@ class ListCollectionIdsRequest(proto.Message):
             [ListCollectionIdsResponse][google.firestore.v1.ListCollectionIdsResponse].
     """
 
-    parent = proto.Field(proto.STRING, number=1)
-
-    page_size = proto.Field(proto.INT32, number=2)
-
-    page_token = proto.Field(proto.STRING, number=3)
+    parent = proto.Field(proto.STRING, number=1,)
+    page_size = proto.Field(proto.INT32, number=2,)
+    page_token = proto.Field(proto.STRING, number=3,)
 
 
 class ListCollectionIdsResponse(proto.Message):
@@ -1023,9 +945,8 @@ class ListCollectionIdsResponse(proto.Message):
     def raw_page(self):
         return self
 
-    collection_ids = proto.RepeatedField(proto.STRING, number=1)
-
-    next_page_token = proto.Field(proto.STRING, number=2)
+    collection_ids = proto.RepeatedField(proto.STRING, number=1,)
+    next_page_token = proto.Field(proto.STRING, number=2,)
 
 
 class BatchWriteRequest(proto.Message):
@@ -1046,11 +967,9 @@ class BatchWriteRequest(proto.Message):
             Labels associated with this batch write.
     """
 
-    database = proto.Field(proto.STRING, number=1)
-
+    database = proto.Field(proto.STRING, number=1,)
     writes = proto.RepeatedField(proto.MESSAGE, number=2, message=write.Write,)
-
-    labels = proto.MapField(proto.STRING, proto.STRING, number=3)
+    labels = proto.MapField(proto.STRING, proto.STRING, number=3,)
 
 
 class BatchWriteResponse(proto.Message):
@@ -1071,8 +990,7 @@ class BatchWriteResponse(proto.Message):
     write_results = proto.RepeatedField(
         proto.MESSAGE, number=1, message=write.WriteResult,
     )
-
-    status = proto.RepeatedField(proto.MESSAGE, number=2, message=gr_status.Status,)
+    status = proto.RepeatedField(proto.MESSAGE, number=2, message=status_pb2.Status,)
 
 
 __all__ = tuple(sorted(__protobuf__.manifest))
